@@ -27,6 +27,7 @@
 # - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
 # - Gabriele Fronze' <gfronze@cern.ch>, 2019
 # - Jaroslav Guenther <jaroslav.guenther@gmail.com>, 2019
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -55,7 +56,7 @@ import zlib
 from logging import getLogger, Formatter
 from logging.handlers import RotatingFileHandler
 from uuid import uuid4 as uuid
-from six import string_types, PY3
+from six import string_types, text_type, PY3
 from xml.etree import ElementTree
 
 try:
@@ -392,11 +393,11 @@ def val_to_space_sep_str(vallist):
     """
     try:
         if isinstance(vallist, list):
-            return u" ".join(vallist)
+            return text_type(" ".join(vallist))
         else:
-            return unicode(vallist)
-    except Exception:
-        return u''
+            return text_type(vallist)
+    except:
+        return text_type('')
 
 
 def date_to_str(date):
@@ -760,27 +761,27 @@ def extract_scope_atlas(did, scopes):
 
 def extract_scope_belleii(did, scopes):
     split_did = did.split('/')
-    if did.find('/belle/MC/') > -1:
-        if len(split_did) > 5:
-            if split_did[4] in ['fab', 'merge1', 'skim']:
+    if did.startswith('/belle/MC/'):
+        if len(split_did) > 4:
+            if split_did[3] in ['fab', 'merge1', 'skim']:
                 return 'mc_tmp', did
         return 'mc', did
-    if did.find('/belle/Raw/') > -1:
+    if did.startswith('/belle/Raw/'):
         return 'raw', did
-    if did.find('/belle/user/') > -1:
-        if len(split_did) > 5:
-            if len(split_did[4]) == 1 and 'user.%s' % (split_did[5]) in scopes:
-                return 'user.%s' % split_did[5], did
+    if did.startswith('/belle/user/'):
         if len(split_did) > 4:
-            if 'user.%s' % (split_did[4]) in scopes:
+            if len(split_did[3]) == 1 and 'user.%s' % (split_did[4]) in scopes:
                 return 'user.%s' % split_did[4], did
+        if len(split_did) > 3:
+            if 'user.%s' % (split_did[3]) in scopes:
+                return 'user.%s' % split_did[3], did
         return 'user', did
-    if did.find('/belle/data/') > -1:
-        if len(split_did) > 5:
-            if split_did[4] in ['fab', 'skim']:
+    if did.startswith('/belle/data/') or did.startswith('/belle/Data/'):
+        if len(split_did) > 4:
+            if split_did[3] in ['fab', 'skim']:
                 return 'data_tmp', did
         return 'data', did
-    if did.find('/belle/ddm/functional_tests/') > -1:
+    if did.startswith('/belle/ddm/functional_tests/') or did.startswith('/belle/ddm/tests/'):
         return 'test', did
     return 'other', did
 
@@ -1279,20 +1280,23 @@ def api_update_return_dict(dictionary):
 
     copied = False  # Avoid side effects from pass by object
 
-    if 'rse_id' in dictionary.keys():
-        if 'rse' not in dictionary.keys():
-            if not copied:
-                dictionary = dictionary.copy()
-                copied = True
+    for rse_str in ['rse', 'src_rse', 'source_rse', 'dest_rse', 'destination_rse']:
+        rse_id_str = '%s_id' % rse_str
+        if rse_id_str in dictionary.keys() and dictionary[rse_id_str] is not None:
+            if rse_str not in dictionary.keys():
+                if not copied:
+                    dictionary = dictionary.copy()
+                    copied = True
+                import rucio.core.rse
+                dictionary[rse_str] = rucio.core.rse.get_rse_name(rse_id=dictionary[rse_id_str])
 
-            import rucio.core.rse
-            dictionary['rse'] = rucio.core.rse.get_rse_name(rse_id=dictionary['rse_id'])
-    if 'account' in dictionary.keys():
+    if 'account' in dictionary.keys() and dictionary['account'] is not None:
         if not copied:
             dictionary = dictionary.copy()
             copied = True
         dictionary['account'] = dictionary['account'].external
-    if 'scope' in dictionary.keys():
+
+    if 'scope' in dictionary.keys() and dictionary['scope'] is not None:
         if not copied:
             dictionary = dictionary.copy()
             copied = True
@@ -1346,3 +1350,33 @@ def query_bunches(query, bunch_by):
     if item_bunch:
         filtered_bunches.append(item_bunch)
     return filtered_bunches
+
+
+class retry:
+    """Retry callable object with configuragle number of attempts"""
+
+    def __init__(self, func, *args, **kwargs):
+        '''
+        :param func: a method that should be executed with retries
+        :param args parametres of the func
+        :param kwargs: key word arguments of the func
+        '''
+        self.func, self.args, self.kwargs = func, args, kwargs
+
+    def __call__(self, mtries=3, logger=None):
+        '''
+        :param mtries: maximum number of attempts to execute the function
+        :param logger: preferred logger
+        '''
+        attempt = mtries
+        while mtries > 1:
+            try:
+                if logger:
+                    logger.debug('{}: Attempt {}'.format(self.func.__name__, mtries - attempt + 1))
+                return self.func(*self.args, **self.kwargs)
+            except Exception as e:
+                if logger:
+                    logger.debug('{}: Attempt failed {}'.format(self.func.__name__, mtries - attempt + 1))
+                    logger.debug(str(e))
+                attempt -= 1
+        return self.func(*self.args, **self.kwargs)
