@@ -24,7 +24,7 @@ from rucio.db.sqla import models
 from rucio.db.sqla.session import transactional_session, read_session
 from rucio.db.sqla.constants import DIDType, RuleGrouping
 from rucio.common.exception import InvalidType
-from rucio.common.types import InternalAccount, InternalScope
+from rucio.common.types import InternalScope, InternalAccount
 from rucio.common.utils import extract_scope
 from rucio.core.did import add_did, attach_dids_to_dids
 from rucio.core.replica import add_replicas
@@ -62,15 +62,13 @@ def add_files(lfns, account, ignore_availability, session=None):
     :session: The session used
     """
     attachments = []
-    account = InternalAccount(account)
     # The list of scopes is necessary for the extract_scope
     scopes = list_scopes(session=session)
     scopes = [scope.external for scope in scopes]
     exist_lfn = []
     for lfn in lfns:
-        # First chekck if the file exists
+        # First check if the file exists
         filename = lfn['lfn']
-        print(filename)
         lfn_scope, _ = extract_scope(filename, scopes)
         lfn_scope = InternalScope(lfn_scope)
         if _exists(lfn_scope, filename):
@@ -91,10 +89,10 @@ def add_files(lfns, account, ignore_availability, session=None):
             add_did(scope=dsn_scope,
                     name=dsn_name,
                     type=DIDType.DATASET,
-                    account=account,
+                    account=InternalAccount(account),
                     statuses=None,
                     meta=None,
-                    rules=[{'copies': 1, 'rse_expression': 'ANY=true', 'weight': None, 'account': account, 'lifetime': None, 'GROUPING': RuleGrouping.NONE}],
+                    rules=[{'copies': 1, 'rse_expression': 'ANY=true', 'weight': None, 'account': InternalAccount(account), 'lifetime': None, 'GROUPING': RuleGrouping.NONE}],
                     lifetime=None,
                     dids=None,
                     rse_id=None,
@@ -118,18 +116,16 @@ def add_files(lfns, account, ignore_availability, session=None):
             files['pfn'] = str(pfn)
         if guid:
             files['meta'] = {'guid': guid}
-        res = add_replicas(rse_id=rse_id,
-                           files=[files],
-                           dataset_meta=None,
-                           account=account,
-                           ignore_availability=ignore_availability,
-                           session=session)
+        add_replicas(rse_id=rse_id,
+                     files=[files],
+                     dataset_meta=None,
+                     account=InternalAccount(account),
+                     ignore_availability=ignore_availability,
+                     session=session)
         attachments.append({'scope': dsn_scope, 'name': dsn_name, 'dids': [{'scope': lfn_scope, 'name': filename}]})
-        print(res)
 
         # Now loop over the ascendants of the dataset and created them
         for lpn in lpns[1:]:
-            print('HERE')
             child_scope, _ = extract_scope(lpn, scopes)
             child_scope = InternalScope(child_scope)
             if (lpn not in exist_lfn) and (not _exists(child_scope, lpn)):
@@ -137,7 +133,7 @@ def add_files(lfns, account, ignore_availability, session=None):
                 add_did(scope=child_scope,
                         name=lpn,
                         type=DIDType.CONTAINER,
-                        account=account,
+                        account=InternalAccount(account),
                         statuses=None,
                         meta=None,
                         rules=None,
@@ -150,9 +146,8 @@ def add_files(lfns, account, ignore_availability, session=None):
                 parent_scope, _ = extract_scope(parent_name, scopes)
                 parent_scope = InternalScope(parent_scope)
                 attachments.append({'scope': parent_scope, 'name': parent_name, 'dids': [{'scope': child_scope, 'name': lpn}]})
-        print(attachments)
     # Finally attach everything
     attach_dids_to_dids(attachments,
-                        account=account,
+                        account=InternalAccount(account),
                         ignore_duplicate=True,
                         session=session)
