@@ -19,6 +19,8 @@
 # - Vincent Garonne <vgaronne@gmail.com>, 2016-2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Brandon White <bjwhite@fnal.gov>, 2019-2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -71,13 +73,14 @@ def re_evaluator(once=False):
     paused_dids = {}  # {(scope, name): datetime}
 
     # Make an initial heartbeat so that all judge-evaluators have the correct worker number on the next try
-    live(executable='rucio-judge-evaluator', hostname=hostname, pid=pid, thread=current_thread, older_than=60 * 30)
+    executable = 'judge-evaluator'
+    live(executable=executable, hostname=hostname, pid=pid, thread=current_thread, older_than=60 * 30)
     graceful_stop.wait(1)
 
     while not graceful_stop.is_set():
         try:
             # heartbeat
-            heartbeat = live(executable='rucio-judge-evaluator', hostname=hostname, pid=pid, thread=current_thread, older_than=60 * 30)
+            heartbeat = live(executable=executable, hostname=hostname, pid=pid, thread=current_thread, older_than=60 * 30)
 
             start = time.time()  # NOQA
 
@@ -85,19 +88,19 @@ def re_evaluator(once=False):
             paused_dids = dict((k, v) for k, v in iteritems(paused_dids) if datetime.utcnow() < v)
 
             # Select a bunch of dids for re evaluation for this worker
-            dids = get_updated_dids(total_workers=heartbeat['nr_threads'] - 1,
+            dids = get_updated_dids(total_workers=heartbeat['nr_threads'],
                                     worker_number=heartbeat['assign_thread'],
                                     limit=100,
                                     blacklisted_dids=[(InternalScope(key[0], fromExternal=False), key[1]) for key in paused_dids])
             logging.debug('re_evaluator[%s/%s] index query time %f fetch size is %d (%d blacklisted)' % (heartbeat['assign_thread'],
-                                                                                                         heartbeat['nr_threads'] - 1,
+                                                                                                         heartbeat['nr_threads'],
                                                                                                          time.time() - start,
                                                                                                          len(dids),
                                                                                                          len([(InternalScope(key[0], fromExternal=False), key[1]) for key in paused_dids])))
 
             # If the list is empty, sent the worker to sleep
             if not dids and not once:
-                logging.debug('re_evaluator[%s/%s] did not get any work (paused_dids=%s)' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, str(len(paused_dids))))
+                logging.debug('re_evaluator[%s/%s] did not get any work (paused_dids=%s)' % (heartbeat['assign_thread'], heartbeat['nr_threads'], str(len(paused_dids))))
                 graceful_stop.wait(30)
             else:
                 done_dids = {}
@@ -109,7 +112,7 @@ def re_evaluator(once=False):
                     did_tag = '%s:%s' % (did.scope.internal, did.name)
                     if did_tag in done_dids:
                         if did.rule_evaluation_action in done_dids[did_tag]:
-                            logging.debug('re_evaluator[%s/%s]: evaluation of %s:%s already done' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, did.scope, did.name))
+                            logging.debug('re_evaluator[%s/%s]: evaluation of %s:%s already done' % (heartbeat['assign_thread'], heartbeat['nr_threads'], did.scope, did.name))
                             delete_updated_did(id=did.id)
                             continue
                     else:
@@ -122,7 +125,7 @@ def re_evaluator(once=False):
                     try:
                         start_time = time.time()
                         re_evaluate_did(scope=did.scope, name=did.name, rule_evaluation_action=did.rule_evaluation_action)
-                        logging.debug('re_evaluator[%s/%s]: evaluation of %s:%s took %f' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, did.scope, did.name, time.time() - start_time))
+                        logging.debug('re_evaluator[%s/%s]: evaluation of %s:%s took %f' % (heartbeat['assign_thread'], heartbeat['nr_threads'], did.scope, did.name, time.time() - start_time))
                         delete_updated_did(id=did.id)
                         done_dids[did_tag].append(did.rule_evaluation_action)
                     except DataIdentifierNotFound as e:
@@ -130,7 +133,7 @@ def re_evaluator(once=False):
                     except (DatabaseException, DatabaseError) as e:
                         if match('.*ORA-00054.*', str(e.args[0])):
                             paused_dids[(did.scope.internal, did.name)] = datetime.utcnow() + timedelta(seconds=randint(60, 600))
-                            logging.warning('re_evaluator[%s/%s]: Locks detected for %s:%s' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, did.scope, did.name))
+                            logging.warning('re_evaluator[%s/%s]: Locks detected for %s:%s' % (heartbeat['assign_thread'], heartbeat['nr_threads'], did.scope, did.name))
                             record_counter('rule.judge.exceptions.LocksDetected')
                         elif match('.*QueuePool.*', str(e.args[0])):
                             logging.warning(traceback.format_exc())
@@ -143,10 +146,10 @@ def re_evaluator(once=False):
                             record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
                     except ReplicationRuleCreationTemporaryFailed as e:
                         record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
-                        logging.warning('re_evaluator[%s/%s]: Replica Creation temporary failed, retrying later for %s:%s' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, did.scope, did.name))
+                        logging.warning('re_evaluator[%s/%s]: Replica Creation temporary failed, retrying later for %s:%s' % (heartbeat['assign_thread'], heartbeat['nr_threads'], did.scope, did.name))
                     except FlushError as e:
                         record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
-                        logging.warning('re_evaluator[%s/%s]: Flush error for %s:%s' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, did.scope, did.name))
+                        logging.warning('re_evaluator[%s/%s]: Flush error for %s:%s' % (heartbeat['assign_thread'], heartbeat['nr_threads'], did.scope, did.name))
         except (DatabaseException, DatabaseError) as e:
             if match('.*QueuePool.*', str(e.args[0])):
                 logging.warning(traceback.format_exc())
@@ -164,7 +167,7 @@ def re_evaluator(once=False):
         if once:
             break
 
-    die(executable='rucio-judge-evaluator', hostname=hostname, pid=pid, thread=current_thread)
+    die(executable=executable, hostname=hostname, pid=pid, thread=current_thread)
 
 
 def stop(signum=None, frame=None):
@@ -180,8 +183,9 @@ def run(once=False, threads=1):
     Starts up the Judge-Eval threads.
     """
 
+    executable = 'judge-evaluator'
     hostname = socket.gethostname()
-    sanity_check(executable='rucio-judge-evaluator', hostname=hostname)
+    sanity_check(executable=executable, hostname=hostname)
 
     if once:
         re_evaluator(once)
