@@ -18,6 +18,8 @@
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2016-2019
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2020
+# - Brandon White <bjwhite@fnal.gov>, 2019-2020-2020
 #
 # PY3K COMPATIBLE
 
@@ -35,7 +37,7 @@ import threading
 import time
 import traceback
 
-from rucio.common.config import config_get
+from rucio.common.config import config_get, config_get_bool
 from rucio.common.exception import (SourceNotFound, DatabaseException, ServiceUnavailable,
                                     RSEAccessDenied, ResourceTemporaryUnavailable)
 from rucio.core import rse as rse_core
@@ -45,6 +47,7 @@ from rucio.core.quarantined_replica import (list_quarantined_replicas,
                                             delete_quarantined_replicas,
                                             list_rses)
 from rucio.rse import rsemanager as rsemgr
+from rucio.core.rse_expression_parser import parse_expression
 
 
 logging.getLogger("requests").setLevel(logging.CRITICAL)
@@ -59,7 +62,7 @@ logging.basicConfig(stream=sys.stdout,
 GRACEFUL_STOP = threading.Event()
 
 
-def reaper(rses=[], worker_number=1, total_workers=1, chunk_size=100, once=False, scheme=None):
+def reaper(rses=[], worker_number=0, total_workers=1, chunk_size=100, once=False, scheme=None):
     """
     Main loop to select and delete files.
 
@@ -89,28 +92,30 @@ def reaper(rses=[], worker_number=1, total_workers=1, chunk_size=100, once=False
             random.shuffle(rses)
             for rse_id in rses:
                 rse = rse_core.get_rse_name(rse_id=rse_id)
-                rse_info = rsemgr.get_rse_info(rse)
                 replicas = list_quarantined_replicas(rse_id=rse_id,
                                                      limit=chunk_size, worker_number=worker_number,
                                                      total_workers=total_workers)
 
-                rse_protocol = rse_core.get_rse_protocols(rse_id=rse_id)
+                rse_info = rsemgr.get_rse_info(rse_id=rse_id)
                 prot = rsemgr.create_protocol(rse_info, 'delete', scheme=scheme)
                 deleted_replicas = []
                 try:
                     prot.connect()
                     for replica in replicas:
                         nothing_to_do = False
+                        scope = ''
+                        if replica['scope']:
+                            scope = replica['scope'].external
                         try:
                             pfn = str(rsemgr.lfns2pfns(rse_settings=rse_info,
-                                                       lfns=[{'scope': replica['scope'].external, 'name': replica['name'], 'path': replica['path']}],
+                                                       lfns=[{'scope': scope, 'name': replica['name'], 'path': replica['path']}],
                                                        operation='delete', scheme=scheme).values()[0])
-                            logging.info('Dark Reaper %s-%s: Deletion ATTEMPT of %s:%s as %s on %s', worker_number, total_workers, replica['scope'], replica['name'], pfn, rse)
+                            logging.info('Dark Reaper %s-%s: Deletion ATTEMPT of %s:%s as %s on %s', worker_number, total_workers, scope, replica['name'], pfn, rse)
                             start = time.time()
                             prot.delete(pfn)
                             duration = time.time() - start
-                            logging.info('Dark Reaper %s-%s: Deletion SUCCESS of %s:%s as %s on %s in %s seconds', worker_number, total_workers, replica['scope'], replica['name'], pfn, rse, duration)
-                            add_message('deletion-done', {'scope': replica['scope'].external,
+                            logging.info('Dark Reaper %s-%s: Deletion SUCCESS of %s:%s as %s on %s in %s seconds', worker_number, total_workers, scope, replica['name'], pfn, rse, duration)
+                            add_message('deletion-done', {'scope': scope,
                                                           'name': replica['name'],
                                                           'rse': rse,
                                                           'rse_id': rse_id,
@@ -121,13 +126,13 @@ def reaper(rses=[], worker_number=1, total_workers=1, chunk_size=100, once=False
                                                           'protocol': prot.attributes['scheme']})
                             deleted_replicas.append(replica)
                         except SourceNotFound:
-                            err_msg = 'Dark Reaper %s-%s: Deletion NOTFOUND of %s:%s as %s on %s' % (worker_number, total_workers, replica['scope'], replica['name'], pfn, rse)
+                            err_msg = 'Dark Reaper %s-%s: Deletion NOTFOUND of %s:%s as %s on %s' % (worker_number, total_workers, scope, replica['name'], pfn, rse)
                             logging.warning(err_msg)
                             deleted_replicas.append(replica)
                         except (ServiceUnavailable, RSEAccessDenied, ResourceTemporaryUnavailable) as error:
-                            err_msg = 'Dark Reaper %s-%s: Deletion NOACCESS of %s:%s as %s on %s: %s' % (worker_number, total_workers, replica['scope'], replica['name'], pfn, rse, str(error))
+                            err_msg = 'Dark Reaper %s-%s: Deletion NOACCESS of %s:%s as %s on %s: %s' % (worker_number, total_workers, scope, replica['name'], pfn, rse, str(error))
                             logging.warning(err_msg)
-                            add_message('deletion-failed', {'scope': replica['scope'].external,
+                            add_message('deletion-failed', {'scope': scope,
                                                             'name': replica['name'],
                                                             'rse': rse,
                                                             'rse_id': rse_id,
@@ -137,7 +142,7 @@ def reaper(rses=[], worker_number=1, total_workers=1, chunk_size=100, once=False
                                                             'reason': str(error),
                                                             'protocol': prot.attributes['scheme']})
 
-                        except:
+                        except Exception:
                             logging.critical(traceback.format_exc())
                 finally:
                     prot.close()
@@ -156,7 +161,7 @@ def reaper(rses=[], worker_number=1, total_workers=1, chunk_size=100, once=False
 
         except DatabaseException as error:
             logging.warning('Reaper:  %s', str(error))
-        except:
+        except Exception:
             logging.critical(traceback.format_exc())
 
     die(executable=executable, hostname=hostname, pid=pid, thread=thread, hash_executable=hash_executable)
@@ -182,19 +187,38 @@ def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None,
     :param threads_per_worker: Total number of threads created by each worker.
     :param once: If True, only runs one iteration of the main loop.
     :param greedy: If True, delete right away replicas with tombstone.
-    :param rses: List of RSEs the reaper should work against. If empty, it considers all RSEs.
+    :param rses: List of RSEs the reaper should work against. If empty, it considers all RSEs (Single-VO only).
     :param scheme: Force the reaper to use a particular protocol/scheme, e.g., mock.
     :param exclude_rses: RSE expression to exclude RSEs from the Reaper.
     :param include_rses: RSE expression to include RSEs.
     """
     logging.info('main: starting processes')
 
+    all_rses = list_rses()
     if all_rses:
-        rses = list_rses()
-    elif not rses:
-        rses = [rse['id'] for rse in rse_core.list_rses()]
+        rses = all_rses
     else:
-        rses = [rse_core.get_rse_id(rse=rse) for rse in rses]
+        if rses:
+            if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+                logging.warning('Ignoring argument rses, this is only available in a single-vo setup. Please try an RSE Expression with include_rses if it is required.')
+                rses = []
+            else:
+                rses = [rse_core.get_rse_id(rse=rse) for rse in rses]
+                rses = [rse for rse in rses if rse in all_rses]
+        else:
+            rses = all_rses
+
+        if exclude_rses:
+            excluded_rses = [rse['id'] for rse in parse_expression(exclude_rses)]
+            rses = [rse for rse in rses if rse not in excluded_rses]
+
+        if include_rses:
+            included_rses = [rse['id'] for rse in parse_expression(include_rses)]
+            rses = [rse for rse in rses if rse in included_rses]
+
+        if not rses:
+            logging.error('Dark Reaper: No RSEs found. Exiting.')
+            return
 
     threads = []
     for worker in range(total_workers):
