@@ -35,12 +35,13 @@ import threading
 import time
 import traceback
 
-from rucio.common.config import config_get
+from rucio.common.config import config_get, config_get_bool
 from rucio.common.exception import (SourceNotFound, DatabaseException, ServiceUnavailable,
                                     RSEAccessDenied, ResourceTemporaryUnavailable)
 from rucio.core import rse as rse_core
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.message import add_message
+from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.temporary_did import (list_expired_temporary_dids, delete_temporary_dids)
 from rucio.rse import rsemanager as rsemgr
 
@@ -91,7 +92,7 @@ def reaper(rses=[], worker_number=0, total_workers=1, chunk_size=100, once=False
                                                        total_workers=total_workers)
 
                 rse = rse_core.get_rse_name(rse_id=rse_id)
-                rse_info = rsemgr.get_rse_info(rse)
+                rse_info = rsemgr.get_rse_info(rse_id=rse_id)
                 rse_protocol = rse_core.get_rse_protocols(rse_id=rse_id)
                 prot = rsemgr.create_protocol(rse_info, 'delete', scheme=scheme)
                 deleted_replicas = []
@@ -181,17 +182,38 @@ def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None,
     :param threads_per_worker: Total number of threads created by each worker.
     :param once: If True, only runs one iteration of the main loop.
     :param greedy: If True, delete right away replicas with tombstone.
-    :param rses: List of RSEs the reaper should work against. If empty, it considers all RSEs.
+    :param rses: List of RSEs the reaper should work against. If empty, it considers all RSEs. (Single-VO only)
     :param scheme: Force the reaper to use a particular protocol/scheme, e.g., mock.
     :param exclude_rses: RSE expression to exclude RSEs from the Reaper.
     :param include_rses: RSE expression to include RSEs.
     """
     logging.info('main: starting processes')
 
+    all_rses = rse_core.list_rses()
     if all_rses:
-        rses = [rse['id'] for rse in rse_core.list_rses()]
+        rses = all_rses
     else:
-        rses = [rse_core.get_rse_id(rse=rse) for rse in rses]
+        if rses:
+            if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+                logging.warning('Ignoring argument rses, this is only available in a single-vo setup. Please try an RSE Expression with include_rses if it is required.')
+                rses = []
+            else:
+                rses = [rse_core.get_rse_id(rse=rse) for rse in rses]
+                rses = [rse for rse in rses if rse in all_rses]
+        else:
+            rses = all_rses
+
+        if exclude_rses:
+            excluded_rses = [rse['id'] for rse in parse_expression(exclude_rses)]
+            rses = [rse for rse in rses if rse not in excluded_rses]
+
+        if include_rses:
+            included_rses = [rse['id'] for rse in parse_expression(include_rses)]
+            rses = [rse for rse in rses if rse in included_rses]
+
+        if not rses:
+            logging.error('Dark Reaper: No RSEs found. Exiting.')
+            return
 
     threads = []
     for worker in range(total_workers):
