@@ -1,4 +1,4 @@
-# Copyright 2015-2018 CERN for the benefit of the ATLAS collaboration.
+# Copyright 2015-2020 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,11 +17,13 @@
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2015
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
-from nose.tools import assert_equal, assert_true
+from nose.tools import assert_equal, assert_true, assert_not_equal
 
 from rucio.core.did import attach_dids, add_did, add_dids
 from rucio.core.replica import list_datasets_per_rse, update_collection_replica, get_cleaned_updated_collection_replicas, delete_replicas, add_replicas
@@ -29,6 +31,7 @@ from rucio.core.rse import add_rse, del_rse, add_protocol, get_rse_id
 from rucio.client.didclient import DIDClient
 from rucio.client.replicaclient import ReplicaClient
 from rucio.client.ruleclient import RuleClient
+from rucio.common.config import config_get, config_get_bool
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid
 from rucio.db.sqla import session, models, constants
@@ -36,6 +39,11 @@ from rucio.tests.common import rse_name_generator
 
 
 class TestDatasetReplicaClient:
+    def setup(self):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            self.vo = {}
 
     def test_list_dataset_replicas(self):
         """ REPLICA (CLIENT): List dataset replicas."""
@@ -52,6 +60,28 @@ class TestDatasetReplicaClient:
         replicas = [r for r in replica_client.list_dataset_replicas(scope=scope, name=dataset)]
         assert_equal(len(replicas), 1)
 
+    def test_list_dataset_replicas_bulk(self):
+        """ REPLICA (CLIENT): List dataset replicas bulk."""
+        replica_client = ReplicaClient()
+        rule_client = RuleClient()
+        did_client = DIDClient()
+        scope = 'mock'
+        did1 = {'scope': scope, 'name': 'dataset_' + str(generate_uuid())}
+        did_client.add_dataset(**did1)
+        did2 = {'scope': scope, 'name': 'dataset_' + str(generate_uuid())}
+        did_client.add_dataset(**did2)
+        dids = [did1, did2]
+        rule_client.add_replication_rule(dids=dids,
+                                         account='root', copies=1, rse_expression='MOCK',
+                                         grouping='DATASET')
+        replicas = list(replica_client.list_dataset_replicas_bulk(dids=dids))
+
+        assert_equal(len(replicas), 2)
+        for did in dids:
+            def replica_contains_did(rep):
+                return all(map(lambda k: k in rep and did[k] == rep[k], did))
+            assert_true(any(map(replica_contains_did, replicas)), "%s must be in returned replicas" % (did, ))
+
     def test_list_datasets_per_rse(self):
         """ REPLICA (CLIENT): List datasets in RSE."""
         rule_client = RuleClient()
@@ -63,8 +93,9 @@ class TestDatasetReplicaClient:
         rule_client.add_replication_rule(dids=[{'scope': scope, 'name': dataset}],
                                          account='root', copies=1, rse_expression='MOCK',
                                          grouping='DATASET')
-        replicas = [r for r in list_datasets_per_rse(rse_id=get_rse_id(rse='MOCK'), filters={'scope': InternalScope('mock'), 'name': 'data*'})]
-        assert(replicas != [])
+        replicas = [r for r in list_datasets_per_rse(rse_id=get_rse_id(rse='MOCK', **self.vo),
+                                                     filters={'scope': InternalScope(scope, **self.vo), 'name': 'data*'})]
+        assert_not_equal(replicas, [])
 
     def test_list_dataset_replicas_archive(self):
         """ REPLICA (CLIENT): List dataset replicas with archives. """
@@ -76,7 +107,7 @@ class TestDatasetReplicaClient:
         scope = 'mock'
 
         rse = 'APERTURE_%s' % rse_name_generator()
-        rse_id = add_rse(rse)
+        rse_id = add_rse(rse, **self.vo)
         add_protocol(rse_id=rse_id, parameter={'scheme': 'root',
                                                'hostname': 'root.aperture.com',
                                                'port': 1409,
@@ -87,7 +118,7 @@ class TestDatasetReplicaClient:
                                                    'wan': {'read': 1, 'write': 1, 'delete': 1}}})
 
         rse2 = 'BLACKMESA_%s' % rse_name_generator()
-        rse2_id = add_rse(rse2)
+        rse2_id = add_rse(rse2, **self.vo)
         add_protocol(rse_id=rse2_id, parameter={'scheme': 'root',
                                                 'hostname': 'root.blackmesa.com',
                                                 'port': 1409,
@@ -134,12 +165,17 @@ class TestDatasetReplicaClient:
 
 class TestDatasetReplicaUpdate:
     def setUp(self):
-        self.scope = InternalScope('mock')
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            self.vo = {}
+
+        self.scope = InternalScope('mock', **self.vo)
         self.rse = 'MOCK4'
         self.rse2 = 'MOCK3'
-        self.account = InternalAccount('root')
-        self.rse_id = get_rse_id(self.rse)
-        self.rse2_id = get_rse_id(self.rse2)
+        self.account = InternalAccount('root', **self.vo)
+        self.rse_id = get_rse_id(self.rse, **self.vo)
+        self.rse2_id = get_rse_id(self.rse2, **self.vo)
         self.db_session = session.get_session()
 
     def tearDown(self):
@@ -182,7 +218,8 @@ class TestDatasetReplicaUpdate:
         add_replicas(rse_id=self.rse_id, files=files, account=self.account, session=self.db_session)
         add_did(scope=self.scope, name=dataset_name, type=constants.DIDType.DATASET, account=self.account, session=self.db_session)
         attach_dids(scope=self.scope, name=dataset_name, dids=files, account=self.account, session=self.db_session)
-        models.CollectionReplica(rse_id=self.rse_id, scope=self.scope, state=constants.ReplicaState.AVAILABLE, name=dataset_name, did_type=constants.DIDType.DATASET, bytes=len(files) * file_size, length=len(files)).save(session=self.db_session)
+        models.CollectionReplica(rse_id=self.rse_id, scope=self.scope, state=constants.ReplicaState.AVAILABLE, name=dataset_name, did_type=constants.DIDType.DATASET, bytes=len(files) * file_size, length=len(files), available_replicas_cnt=0)\
+              .save(session=self.db_session)
 
         # Update request with rse id
         # First update -> dataset replica should be available
