@@ -16,6 +16,8 @@
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
 # - Tobias Wegner <twegner@cern.ch>, 2019
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2019
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -28,6 +30,7 @@ import os.path
 from rucio.client.client import Client
 from rucio.client.downloadclient import DownloadClient
 from rucio.client.uploadclient import UploadClient
+from rucio.common.config import config_get, config_get_bool
 from rucio.common.utils import generate_uuid
 from rucio.tests.common import file_generator
 
@@ -35,6 +38,11 @@ from rucio.tests.common import file_generator
 class TestDownloadClient(object):
 
     def setup(self):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            self.vo = {}
+
         logger = logging.getLogger('dlul_client')
         logger.addHandler(logging.StreamHandler())
         logger.setLevel(logging.DEBUG)
