@@ -14,6 +14,9 @@
 #
 # Authors:
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -21,53 +24,278 @@ from datetime import datetime
 from nose.tools import assert_equal
 from paste.fixture import TestApp
 
+from rucio.common.config import config_get, config_get_bool
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid, parse_response
+from rucio.core.config import set as config_set
 from rucio.core.did import attach_dids, add_did
+# from rucio.core.distance import add_distance
 from rucio.core.replica import add_replica
-from rucio.core.request import release_all_waiting_requests, queue_requests, get_request_by_did, release_waiting_requests_per_free_volume, release_waiting_requests_grouped_fifo, release_waiting_requests_fifo, list_requests
+from rucio.core.request import release_all_waiting_requests, queue_requests, get_request_by_did, release_waiting_requests_per_free_volume,\
+    release_waiting_requests_grouped_fifo, release_waiting_requests_fifo, list_requests, release_waiting_requests_per_deadline
 from rucio.core.rse import get_rse_id, set_rse_transfer_limits, add_rse_attribute
 from rucio.db.sqla import session, models, constants
 from rucio.web.rest.authentication import APP as auth_app
 from rucio.web.rest.request import APP as request_app
 
 
-class TestRequestCore(object):
+class TestRequestCoreQueue(object):
 
     @classmethod
     def setUpClass(cls):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            cls.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            cls.vo = {}
+
         cls.db_session = session.get_session()
         cls.dialect = cls.db_session.bind.dialect.name
         cls.dest_rse = 'MOCK'
         cls.dest_rse2 = 'MOCK2'
         cls.source_rse = 'MOCK4'
         cls.source_rse2 = 'MOCK5'
-        cls.dest_rse_id = get_rse_id(cls.dest_rse)
-        cls.dest_rse_id2 = get_rse_id(cls.dest_rse2)
-        cls.source_rse_id = get_rse_id(cls.source_rse)
-        cls.source_rse_id2 = get_rse_id(cls.source_rse2)
-        cls.scope = InternalScope('mock')
-        cls.account = InternalAccount('root')
+        cls.dest_rse_id = get_rse_id(cls.dest_rse, **cls.vo)
+        cls.dest_rse_id2 = get_rse_id(cls.dest_rse2, **cls.vo)
+        cls.source_rse_id = get_rse_id(cls.source_rse, **cls.vo)
+        cls.source_rse_id2 = get_rse_id(cls.source_rse2, **cls.vo)
+        cls.scope = InternalScope('mock', **cls.vo)
+        cls.account = InternalAccount('root', **cls.vo)
+        cls.source_rse_id2 = get_rse_id(cls.source_rse2, **cls.vo)
         cls.user_activity = 'User Subscription'
-        cls.all_activities = 'all_activities'
 
     def setUp(self):
         self.db_session.query(models.Source).delete()
         self.db_session.query(models.Request).delete()
+        self.db_session.query(models.RSETransferLimit).delete()
+        self.db_session.query(models.Distance).delete()
+        self.db_session.query(models.Config).delete()
         self.db_session.commit()
+
+    def tearDown(self):
+        self.db_session.commit()
+
+    def test_queue_requests_state_no_throttler(self):
+        """ REQUEST (CORE): queue requests with default throttler mode (None). """
+        name = generate_uuid()
+        name2 = generate_uuid()
+        name3 = generate_uuid()
+        add_replica(self.source_rse_id, self.scope, name, 1, self.account, session=self.db_session)
+        add_replica(self.source_rse_id2, self.scope, name2, 1, self.account, session=self.db_session)
+        add_replica(self.source_rse_id, self.scope, name3, 1, self.account, session=self.db_session)
+
+        set_rse_transfer_limits(self.dest_rse_id, self.user_activity, max_transfers=1, session=self.db_session)
+        set_rse_transfer_limits(self.dest_rse_id2, self.user_activity, max_transfers=1, session=self.db_session)
+        set_rse_transfer_limits(self.source_rse_id, self.user_activity, max_transfers=1, session=self.db_session)
+        set_rse_transfer_limits(self.source_rse_id2, self.user_activity, max_transfers=1, session=self.db_session)
+
+        requests = [{
+            'dest_rse_id': self.dest_rse_id,
+            'src_rse_id': self.source_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'request_id': generate_uuid(),
+            'name': name,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'requested_at': datetime.now().replace(year=2015),
+            'attributes': {
+                'activity': self.user_activity,
+                'bytes': 10,
+                'md5': '',
+                'adler32': ''
+            }
+        }, {
+            'dest_rse_id': self.dest_rse_id,
+            'src_rse_id': self.source_rse_id2,
+            'request_type': constants.RequestType.TRANSFER,
+            'request_id': generate_uuid(),
+            'name': name2,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'requested_at': datetime.now().replace(year=2015),
+            'attributes': {
+                'activity': 'unknown',
+                'bytes': 10,
+                'md5': '',
+                'adler32': ''
+            }
+        }, {
+            'dest_rse_id': self.dest_rse_id2,
+            'src_rse_id': self.source_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'request_id': generate_uuid(),
+            'name': name3,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'requested_at': datetime.now().replace(year=2015),
+            'attributes': {
+                'activity': self.user_activity,
+                'bytes': 10,
+                'md5': '',
+                'adler32': ''
+            }
+        }]
+        queue_requests(requests, session=self.db_session)
+        request = get_request_by_did(self.scope, name, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.QUEUED)
+        request = get_request_by_did(self.scope, name2, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.QUEUED)
+        request = get_request_by_did(self.scope, name3, self.dest_rse_id2, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.QUEUED)
+
+    # def test_queue_requests_source_rse(self):
+    #     """ REQUEST (CORE): queue requests and select correct source RSE. """
+    #     # test correct selection of source RSE
+    #     name = generate_uuid()
+    #     size = 8
+    #     add_replica(self.source_rse_id, self.scope, name, size, self.account, session=self.db_session)
+    #     add_replica(self.source_rse_id2, self.scope, name, size, self.account, session=self.db_session)
+    #     add_distance(self.source_rse_id, self.dest_rse_id, 1, session=self.db_session)
+    #     add_distance(self.source_rse_id2, self.dest_rse_id, 2, session=self.db_session)
+    #     requests = [{
+    #         'dest_rse_id': self.dest_rse_id,
+    #         'request_type': constants.RequestType.TRANSFER,
+    #         'request_id': generate_uuid(),
+    #         'name': name,
+    #         'scope': self.scope,
+    #         'rule_id': generate_uuid(),
+    #         'retry_count': 1,
+    #         'requested_at': datetime.now().replace(year=2015),
+    #         'attributes': {
+    #             'activity': self.user_activity,
+    #             'bytes': size,
+    #             'md5': '',
+    #             'adler32': ''
+    #         }
+    #     }]
+    #     queue_requests(requests, session=self.db_session)
+    #     request = get_request_by_did(self.scope, name, self.dest_rse_id, session=self.db_session)
+    #     # select source RSE with smallest distance
+    #     assert_equal(request['source_rse_id'], self.source_rse_id)
+    #     assert_equal(request['name'], name)
+    #     assert_equal(request['scope'], self.scope)
+    #     assert_equal(request['state'], constants.RequestState.QUEUED)
+
+    def test_queue_requests_state_1(self):
+        """ REQUEST (CORE): queue requests and set correct request state. """
+        # test correct request state depending on throttler mode
+        config_set('throttler', 'mode', 'DEST_PER_ACT', session=self.db_session)
+        set_rse_transfer_limits(self.dest_rse_id, self.user_activity, max_transfers=1, session=self.db_session)
+        size = 1
+        name = generate_uuid()
+        add_replica(self.source_rse_id2, self.scope, name, size, self.account, session=self.db_session)
+        name2 = generate_uuid()
+        add_replica(self.source_rse_id2, self.scope, name2, size, self.account, session=self.db_session)
+        requests = [{
+            'dest_rse_id': self.dest_rse_id,
+            'source_rse_id': self.source_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'request_id': generate_uuid(),
+            'name': name,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'requested_at': datetime.now().replace(year=2015),
+            'attributes': {
+                'activity': self.user_activity,
+                'bytes': size,
+                'md5': '',
+                'adler32': ''
+            }
+        }, {
+            'dest_rse_id': self.dest_rse_id,
+            'source_rse_id': self.source_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'request_id': generate_uuid(),
+            'name': name2,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'requested_at': datetime.now().replace(year=2015),
+            'attributes': {
+                'activity': 'Activity without limit',
+                'bytes': size,
+                'md5': '',
+                'adler32': ''
+            }
+        }]
+        queue_requests(requests, session=self.db_session)
+        request = get_request_by_did(self.scope, name, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.WAITING)
+        request = get_request_by_did(self.scope, name2, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.QUEUED)
+
+    def test_queue_requests_state_2(self):
+        """ REQUEST (CORE): queue requests and set correct request state. """
+        config_set('throttler', 'mode', 'SRC_PER_ACT', session=self.db_session)
+        size = 1
+        name = generate_uuid()
+        add_replica(self.source_rse_id2, self.scope, name, size, self.account, session=self.db_session)
+        name2 = generate_uuid()
+        add_replica(self.source_rse_id2, self.scope, name2, size, self.account, session=self.db_session)
+        requests = [{
+            'dest_rse_id': self.dest_rse_id,
+            'source_rse_id': self.source_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'request_id': generate_uuid(),
+            'name': name,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'requested_at': datetime.now().replace(year=2015),
+            'attributes': {
+                'activity': self.user_activity,
+                'bytes': size,
+                'md5': '',
+                'adler32': ''
+            }
+        }]
+        queue_requests(requests, session=self.db_session)
+        request = get_request_by_did(self.scope, name, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.QUEUED)
+
+
+class TestRequestCoreRelease(object):
+
+    @classmethod
+    def setUpClass(cls):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            cls.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            cls.vo = {}
+
+        cls.db_session = session.get_session()
+        cls.dialect = cls.db_session.bind.dialect.name
+        cls.dest_rse = 'MOCK'
+        cls.source_rse = 'MOCK4'
+        cls.source_rse2 = 'MOCK5'
+        cls.dest_rse_id = get_rse_id(cls.dest_rse, **cls.vo)
+        cls.source_rse_id = get_rse_id(cls.source_rse, **cls.vo)
+        cls.source_rse_id2 = get_rse_id(cls.source_rse2, **cls.vo)
+        cls.scope = InternalScope('mock', **cls.vo)
+        cls.account = InternalAccount('root', **cls.vo)
+        cls.user_activity = 'User Subscription'
+        cls.all_activities = 'all_activities'
+
+    def setUp(self):
+        self.db_session.query(models.Request).delete()
+        self.db_session.query(models.RSETransferLimit).delete()
+        self.db_session.query(models.Distance).delete()
+        self.db_session.query(models.Config).delete()
         # set transfer limits to put requests in waiting state
         set_rse_transfer_limits(self.dest_rse_id, self.user_activity, max_transfers=1, session=self.db_session)
         set_rse_transfer_limits(self.dest_rse_id, self.all_activities, max_transfers=1, session=self.db_session)
         set_rse_transfer_limits(self.dest_rse_id, 'ignore', max_transfers=1, session=self.db_session)
+        config_set('throttler', 'mode', 'DEST_PER_ACT', session=self.db_session)
+        self.db_session.commit()
 
     def tearDown(self):
         self.db_session.commit()
 
     def test_release_waiting_requests_per_free_volume(self):
         """ REQUEST (CORE): release waiting requests that fit grouped in available volume."""
-        if self.dialect == 'mysql':
-            return True
-
         # release unattached requests that fit in available volume with respect to already submitted transfers
         name1 = generate_uuid()
         add_replica(self.source_rse_id, self.scope, name1, 1, self.account, session=self.db_session)
@@ -78,9 +306,10 @@ class TestRequestCore(object):
         request = models.Request(dest_rse_id=self.dest_rse_id, bytes=2, activity=self.all_activities, state=constants.RequestState.SUBMITTED)
         request.save(session=self.db_session)
         volume = 10
-        set_rse_transfer_limits(self.dest_rse_id, 'all_activities', volume=volume, max_transfers=1, session=self.db_session)
+        set_rse_transfer_limits(self.dest_rse_id, self.all_activities, volume=volume, max_transfers=1, session=self.db_session)
         requests = [{
             'dest_rse_id': self.dest_rse_id,
+            'source_rse_id': self.source_rse_id,
             'request_type': constants.RequestType.TRANSFER,
             'request_id': generate_uuid(),
             'name': name1,
@@ -96,6 +325,7 @@ class TestRequestCore(object):
             }
         }, {
             'dest_rse_id': self.dest_rse_id,
+            'source_rse_id': self.source_rse_id,
             'request_type': constants.RequestType.TRANSFER,
             'request_id': generate_uuid(),
             'name': name2,
@@ -111,6 +341,7 @@ class TestRequestCore(object):
             }
         }, {
             'dest_rse_id': self.dest_rse_id,
+            'source_rse_id': self.source_rse_id,
             'request_type': constants.RequestType.TRANSFER,
             'request_id': generate_uuid(),
             'name': name3,
@@ -157,9 +388,10 @@ class TestRequestCore(object):
         request = models.Request(dest_rse_id=self.dest_rse_id, bytes=2, activity=self.all_activities, state=constants.RequestState.SUBMITTED)
         request.save(session=self.db_session)
         volume = 10
-        set_rse_transfer_limits(self.dest_rse_id, 'all_activities', volume=volume, max_transfers=1, session=self.db_session)
+        set_rse_transfer_limits(self.dest_rse_id, self.all_activities, volume=volume, max_transfers=1, session=self.db_session)
         requests = [{
             'dest_rse_id': self.dest_rse_id,
+            'source_rse_id': self.source_rse_id,
             'request_type': constants.RequestType.TRANSFER,
             'request_id': generate_uuid(),
             'name': name1,
@@ -168,13 +400,14 @@ class TestRequestCore(object):
             'retry_count': 1,
             'requested_at': datetime.now().replace(year=2015),
             'attributes': {
-                'activity': 'User Subscription',
+                'activity': self.user_activity,
                 'bytes': 6,
                 'md5': '',
                 'adler32': ''
             }
         }, {
             'dest_rse_id': self.dest_rse_id,
+            'source_rse_id': self.source_rse_id,
             'request_type': constants.RequestType.TRANSFER,
             'request_id': generate_uuid(),
             'name': name2,
@@ -183,13 +416,14 @@ class TestRequestCore(object):
             'rule_id': generate_uuid(),
             'retry_count': 1,
             'attributes': {
-                'activity': 'User Subscription',
+                'activity': self.user_activity,
                 'bytes': 2,
                 'md5': '',
                 'adler32': ''
             }
         }, {
             'dest_rse_id': self.dest_rse_id,
+            'source_rse_id': self.source_rse_id,
             'request_type': constants.RequestType.TRANSFER,
             'request_id': generate_uuid(),
             'name': name3,
@@ -198,13 +432,14 @@ class TestRequestCore(object):
             'retry_count': 1,
             'requested_at': datetime.now().replace(year=2000),
             'attributes': {
-                'activity': 'User Subscription',
+                'activity': self.user_activity,
                 'bytes': 10,
                 'md5': '',
                 'adler32': ''
             }
         }, {
             'dest_rse_id': self.dest_rse_id,
+            'source_rse_id': self.source_rse_id,
             'request_type': constants.RequestType.TRANSFER,
             'request_id': generate_uuid(),
             'name': name4,
@@ -213,7 +448,7 @@ class TestRequestCore(object):
             'retry_count': 1,
             'requested_at': datetime.now().replace(year=2030),
             'attributes': {
-                'activity': 'User Subscription',
+                'activity': self.user_activity,
                 'bytes': 2,
                 'md5': '',
                 'adler32': ''
@@ -238,9 +473,10 @@ class TestRequestCore(object):
         name1 = generate_uuid()
         add_replica(self.dest_rse_id, self.scope, name1, 1, self.account, session=self.db_session)
         volume = 0
-        set_rse_transfer_limits(self.dest_rse_id, 'all_activities', volume=volume, max_transfers=1, session=self.db_session)
+        set_rse_transfer_limits(self.dest_rse_id, self.all_activities, volume=volume, max_transfers=1, session=self.db_session)
         requests = [{
             'dest_rse_id': self.dest_rse_id,
+            'source_rse_id': self.source_rse_id,
             'request_type': constants.RequestType.TRANSFER,
             'request_id': generate_uuid(),
             'name': name1,
@@ -249,7 +485,7 @@ class TestRequestCore(object):
             'retry_count': 1,
             'requested_at': datetime.now().replace(year=2015),
             'attributes': {
-                'activity': 'User Subscription',
+                'activity': self.user_activity,
                 'bytes': 8,
                 'md5': '',
                 'adler32': ''
@@ -263,10 +499,7 @@ class TestRequestCore(object):
 
     def test_release_waiting_requests_grouped_fifo(self):
         """ REQUEST (CORE): release waiting requests based on grouped FIFO. """
-        if self.dialect == 'mysql':
-            return True
-
-        # set max_volume to 0 to check first without releasing extra requests
+        # set volume and deadline to 0 to check first without releasing extra requests
         set_rse_transfer_limits(self.dest_rse_id, self.all_activities, volume=0, max_transfers=1, session=self.db_session)
 
         # one request with an unattached DID -> one request should be released
@@ -291,7 +524,7 @@ class TestRequestCore(object):
             }
         }]
         queue_requests(requests, session=self.db_session)
-        release_waiting_requests_grouped_fifo(self.dest_rse_id, count=1, session=self.db_session)
+        release_waiting_requests_grouped_fifo(self.dest_rse_id, count=1, volume=0, deadline=0, session=self.db_session)
         request = get_request_by_did(self.scope, name, self.dest_rse_id, session=self.db_session)
         assert_equal(request['state'], constants.RequestState.QUEUED)
 
@@ -320,7 +553,7 @@ class TestRequestCore(object):
             }
         }]
         queue_requests(requests, session=self.db_session)
-        release_waiting_requests_grouped_fifo(self.dest_rse_id, count=1, session=self.db_session)
+        release_waiting_requests_grouped_fifo(self.dest_rse_id, count=1, volume=0, deadline=0, session=self.db_session)
         request = get_request_by_did(self.scope, name, self.dest_rse_id, session=self.db_session)
         assert_equal(request['state'], constants.RequestState.QUEUED)
 
@@ -426,7 +659,7 @@ class TestRequestCore(object):
             }
         }]
         queue_requests(requests, session=self.db_session)
-        release_waiting_requests_grouped_fifo(self.dest_rse_id, count=1, session=self.db_session)
+        release_waiting_requests_grouped_fifo(self.dest_rse_id, count=1, deadline=0, volume=0, session=self.db_session)
         request_1 = get_request_by_did(self.scope, name1, self.dest_rse_id, session=self.db_session)
         assert_equal(request_1['state'], constants.RequestState.QUEUED)
         request_2 = get_request_by_did(self.scope, name2, self.dest_rse_id, session=self.db_session)
@@ -524,7 +757,7 @@ class TestRequestCore(object):
         }]
 
         queue_requests(requests, session=self.db_session)
-        amount_updated_requests = release_waiting_requests_grouped_fifo(self.dest_rse_id, count=1, session=self.db_session)
+        amount_updated_requests = release_waiting_requests_grouped_fifo(self.dest_rse_id, count=1, deadline=0, volume=10, session=self.db_session)
         assert_equal(amount_updated_requests, 3)
         # released because it got requested first
         request_1 = get_request_by_did(self.scope, name1, self.dest_rse_id, session=self.db_session)
@@ -628,7 +861,7 @@ class TestRequestCore(object):
         }]
 
         queue_requests(requests, session=self.db_session)
-        release_waiting_requests_grouped_fifo(self.dest_rse_id, count=1, session=self.db_session)
+        release_waiting_requests_grouped_fifo(self.dest_rse_id, count=1, deadline=0, volume=5, session=self.db_session)
         # released because it got requested first
         request_1 = get_request_by_did(self.scope, name1, self.dest_rse_id, session=self.db_session)
         assert_equal(request_1['state'], constants.RequestState.QUEUED)
@@ -641,6 +874,57 @@ class TestRequestCore(object):
         request_4 = get_request_by_did(self.scope, name4, self.dest_rse_id, session=self.db_session)
         assert_equal(request_4['state'], constants.RequestState.WAITING)
 
+        # with deadline check -> release 0 requests -> 1 request should be released nonetheless
+        self.db_session.query(models.Request).delete()
+        self.db_session.commit()
+        name1 = generate_uuid()
+        add_replica(self.source_rse_id, self.scope, name1, 1, self.account)
+        name2 = generate_uuid()
+        add_replica(self.source_rse_id, self.scope, name2, 1, self.account)
+        current_hour = datetime.now().hour
+        requests = [{
+            'source_rse_id': self.source_rse_id,
+            'dest_rse_id': self.dest_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'requested_at': datetime.now().replace(hour=current_hour - 2),
+            'request_id': generate_uuid(),
+            'name': name1,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'attributes': {
+                'activity': self.user_activity,
+                'bytes': 1,
+                'md5': '',
+                'adler32': ''
+            }
+        }, {
+            'source_rse_id': self.source_rse_id,
+            'dest_rse_id': self.dest_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'requested_at': datetime.now(),
+            'request_id': generate_uuid(),
+            'name': name2,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'attributes': {
+                'activity': self.user_activity,
+                'bytes': 1,
+                'md5': '',
+                'adler32': ''
+            }
+        }]
+        queue_requests(requests, session=self.db_session)
+        request = get_request_by_did(self.scope, name1, self.dest_rse_id, session=self.db_session)
+        release_waiting_requests_grouped_fifo(self.source_rse_id, count=0, deadline=1, volume=0, session=self.db_session)
+        # queued because of deadline
+        request = get_request_by_did(self.scope, name1, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.QUEUED)
+        # waiting because count=0
+        request = get_request_by_did(self.scope, name2, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.WAITING)
+
     def test_release_waiting_requests_fifo(self):
         """ REQUEST (CORE): release waiting requests based on FIFO. """
         # without account and activity check
@@ -745,7 +1029,7 @@ class TestRequestCore(object):
             'scope': self.scope,
             'rule_id': generate_uuid(),
             'retry_count': 1,
-            'account': InternalAccount('jdoe'),
+            'account': InternalAccount('jdoe', **self.vo),
             'attributes': {
                 'activity': self.user_activity,
                 'bytes': 1,
@@ -798,7 +1082,7 @@ class TestRequestCore(object):
             'retry_count': 1,
             'requested_at': datetime.now().replace(year=2018),
             'attributes': {
-                'activity': 'User Subscription',
+                'activity': self.user_activity,
                 'bytes': 1,
                 'md5': '',
                 'adler32': ''
@@ -814,7 +1098,7 @@ class TestRequestCore(object):
             'rule_id': generate_uuid(),
             'retry_count': 1,
             'attributes': {
-                'activity': 'User Subscription',
+                'activity': self.user_activity,
                 'bytes': 1,
                 'md5': '',
                 'adler32': ''
@@ -827,6 +1111,156 @@ class TestRequestCore(object):
         request = get_request_by_did(self.scope, name2, self.dest_rse_id, session=self.db_session)
         assert_equal(request['state'], constants.RequestState.QUEUED)
 
+    def test_release_waiting_requests_per_deadline(self):
+        """ REQUEST (CORE): release grouped waiting requests that exceeded waiting time."""
+        # a request that exceeded the maximal waiting time to be released (1 hour) -> release one request -> only the exceeded request should be released
+        set_rse_transfer_limits(self.source_rse_id, activity=self.all_activities, strategy='grouped_fifo', session=self.db_session)
+        name1 = generate_uuid()
+        add_replica(self.source_rse_id, self.scope, name1, 1, self.account, session=self.db_session)
+        name2 = generate_uuid()
+        add_replica(self.source_rse_id, self.scope, name2, 1, self.account, session=self.db_session)
+        current_hour = datetime.now().hour
+        requests = [{
+            'source_rse_id': self.source_rse_id,
+            'dest_rse_id': self.dest_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'requested_at': datetime.now().replace(hour=current_hour - 2),
+            'request_id': generate_uuid(),
+            'name': name1,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'attributes': {
+                'activity': self.user_activity,
+                'bytes': 1,
+                'md5': '',
+                'adler32': ''
+            }
+        }, {
+            'source_rse_id': self.source_rse_id,
+            'dest_rse_id': self.dest_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'requested_at': datetime.now(),
+            'request_id': generate_uuid(),
+            'name': name2,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'attributes': {
+                'activity': self.user_activity,
+                'bytes': 1,
+                'md5': '',
+                'adler32': ''
+            }
+        }]
+        queue_requests(requests, session=self.db_session)
+        request = get_request_by_did(self.scope, name1, self.dest_rse_id, session=self.db_session)
+        release_waiting_requests_per_deadline(self.source_rse_id, deadline=1, session=self.db_session)
+        request = get_request_by_did(self.scope, name1, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.QUEUED)
+        request = get_request_by_did(self.scope, name2, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.WAITING)
+
+        # a request that exceeded the maximal waiting time to be released (1 hour) -> release one request -> release all requests of the same dataset
+        name1 = generate_uuid()
+        add_replica(self.source_rse_id, self.scope, name1, 1, self.account, session=self.db_session)
+        name2 = generate_uuid()
+        add_replica(self.source_rse_id, self.scope, name2, 1, self.account, session=self.db_session)
+        name3 = generate_uuid()
+        add_replica(self.source_rse_id, self.scope, name3, 1, self.account, session=self.db_session)
+        dataset_name = generate_uuid()
+        add_did(self.scope, dataset_name, constants.DIDType.DATASET, self.account, session=self.db_session)
+        attach_dids(self.scope, dataset_name, [{'name': name1, 'scope': self.scope}, {'name': name2, 'scope': self.scope}], self.account, session=self.db_session)
+        current_hour = datetime.now().hour
+        requests = [{
+            'source_rse_id': self.source_rse_id,
+            'dest_rse_id': self.dest_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'requested_at': datetime.now().replace(hour=current_hour - 2),
+            'request_id': generate_uuid(),
+            'name': name1,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'attributes': {
+                'activity': self.user_activity,
+                'bytes': 1,
+                'md5': '',
+                'adler32': ''
+            }
+        }, {
+            'source_rse_id': self.source_rse_id,
+            'dest_rse_id': self.dest_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'requested_at': datetime.now(),
+            'request_id': generate_uuid(),
+            'name': name2,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'attributes': {
+                'activity': self.user_activity,
+                'bytes': 1,
+                'md5': '',
+                'adler32': ''
+            }
+        }, {
+            'source_rse_id': self.source_rse_id,
+            'dest_rse_id': self.dest_rse_id,
+            'request_type': constants.RequestType.TRANSFER,
+            'requested_at': datetime.now(),
+            'request_id': generate_uuid(),
+            'name': name3,
+            'scope': self.scope,
+            'rule_id': generate_uuid(),
+            'retry_count': 1,
+            'attributes': {
+                'activity': self.user_activity,
+                'bytes': 1,
+                'md5': '',
+                'adler32': ''
+            }
+        }]
+        queue_requests(requests, session=self.db_session)
+        release_waiting_requests_per_deadline(self.source_rse_id, deadline=1, session=self.db_session)
+        request = get_request_by_did(self.scope, name1, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.QUEUED)
+        request = get_request_by_did(self.scope, name2, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.QUEUED)
+        request = get_request_by_did(self.scope, name3, self.dest_rse_id, session=self.db_session)
+        assert_equal(request['state'], constants.RequestState.WAITING)
+
+
+class TestRequestCoreList(object):
+
+    @classmethod
+    def setUpClass(cls):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            cls.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            cls.vo = {}
+
+        cls.db_session = session.get_session()
+        cls.dest_rse = 'MOCK'
+        cls.dest_rse2 = 'MOCK2'
+        cls.source_rse = 'MOCK4'
+        cls.source_rse2 = 'MOCK5'
+        cls.dest_rse_id = get_rse_id(cls.dest_rse, **cls.vo)
+        cls.dest_rse_id2 = get_rse_id(cls.dest_rse2, **cls.vo)
+        cls.source_rse_id = get_rse_id(cls.source_rse, **cls.vo)
+        cls.source_rse_id2 = get_rse_id(cls.source_rse2, **cls.vo)
+
+    def setUp(self):
+        self.db_session.query(models.Source).delete()
+        self.db_session.query(models.Request).delete()
+        self.db_session.query(models.RSETransferLimit).delete()
+        self.db_session.query(models.Distance).delete()
+        self.db_session.query(models.Config).delete()
+        self.db_session.commit()
+
+    def tearDown(self):
+        self.db_session.commit()
+
     def test_list_requests(self):
         """ REQUEST (CORE): list requests """
         models.Request(state=constants.RequestState.WAITING, source_rse_id=self.source_rse_id, dest_rse_id=self.dest_rse_id).save(session=self.db_session)
@@ -849,20 +1283,28 @@ class TestRequestREST():
 
     @classmethod
     def setUpClass(cls):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            cls.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+            cls.vo_header = {'X-Rucio-VO': cls.vo['vo']}
+        else:
+            cls.vo = {}
+            cls.vo_header = {}
+
         cls.mw = []
         headers1 = {'X-Rucio-Account': 'root', 'X-Rucio-Username': 'ddmlab', 'X-Rucio-Password': 'secret'}
+        headers1.update(cls.vo_header)
         r1 = TestApp(auth_app.wsgifunc(*cls.mw)).get('/userpass', headers=headers1, expect_errors=True)
         cls.token = str(r1.header('X-Rucio-Auth-Token'))
         cls.source_rse = 'MOCK'
-        cls.source_rse_id = get_rse_id(cls.source_rse)
+        cls.source_rse_id = get_rse_id(cls.source_rse, **cls.vo)
         cls.source_rse2 = 'MOCK2'
-        cls.source_rse_id2 = get_rse_id(cls.source_rse2)
+        cls.source_rse_id2 = get_rse_id(cls.source_rse2, **cls.vo)
         cls.source_rse3 = 'MOCK5'
-        cls.source_rse_id3 = get_rse_id(cls.source_rse3)
+        cls.source_rse_id3 = get_rse_id(cls.source_rse3, **cls.vo)
         cls.dest_rse = 'MOCK3'
-        cls.dest_rse_id = get_rse_id(cls.dest_rse)
+        cls.dest_rse_id = get_rse_id(cls.dest_rse, **cls.vo)
         cls.dest_rse2 = 'MOCK4'
-        cls.dest_rse_id2 = get_rse_id(cls.dest_rse2)
+        cls.dest_rse_id2 = get_rse_id(cls.dest_rse2, **cls.vo)
         cls.db_session = session.get_session()
         cls.source_site = 'SITE1'
         cls.source_site2 = 'SITE2'
@@ -881,18 +1323,20 @@ class TestRequestREST():
 
     def check_correct_api(self, params, expected_requests):
         headers = {'X-Rucio-Type': 'user', 'X-Rucio-Account': 'root', 'X-Rucio-Auth-Token': str(self.token)}
+        headers.update(self.vo_header)
         r = TestApp(request_app.wsgifunc(*self.mw)).get('/list', params=params, headers=headers, expect_errors=True)
         assert_equal(r.status, 200)
         requests = set()
-        for request in r.body.split('\n')[:-1]:
+        for request in r.body.decode().split('\n')[:-1]:
             request = parse_response(request)
             requests.add((request['state'], request['source_rse_id'], request['dest_rse_id'], request['name']))
         assert_equal(requests, expected_requests)
 
     def check_error_api(self, params, exception_class, exception_message, code):
         headers = {'X-Rucio-Type': 'user', 'X-Rucio-Account': 'root', 'X-Rucio-Auth-Token': str(self.token)}
+        headers.update(self.vo_header)
         r = TestApp(request_app.wsgifunc(*self.mw)).get('/list', params=params, headers=headers, expect_errors=True)
-        body = parse_response(r.body)
+        body = parse_response(r.body.decode())
         assert_equal(r.status, code)
         assert_equal(body['ExceptionClass'], exception_class)
         assert_equal(body['ExceptionMessage'], exception_message)
