@@ -22,8 +22,11 @@
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2014
 # - Martin Barisits <martin.barisits@cern.ch>, 2017
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Ruturaj Gujar, <ruturaj.gujar23@gmail.com>, 2019
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019
+# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019, 2020
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -42,13 +45,13 @@ from web import seeother, setcookie, template
 
 from rucio.api.authentication import (get_auth_oidc, get_auth_token_gss,
                                       get_auth_token_saml, get_auth_token_ssh,
-                                      get_auth_token_user_pass,
-                                      get_auth_token_x509,
+                                      get_auth_token_user_pass, get_auth_token_x509,
                                       get_ssh_challenge_token, get_token_oidc,
-                                      redirect_auth_oidc, validate_auth_token)
+                                      redirect_auth_oidc, refresh_cli_auth_token,
+                                      validate_auth_token)
 from rucio.common.config import config_get
 from rucio.common.exception import AccessDenied, IdentityError, RucioException
-from rucio.common.utils import date_to_str, generate_http_error
+from rucio.common.utils import date_to_str, generate_http_error, urlparse
 from rucio.web.rest.common import RucioController, check_accept_header_wrapper
 
 # Extra modules: Only imported if available
@@ -79,6 +82,7 @@ URLS = (
     '/oidc_token', 'TokenOIDC',
     '/oidc_code', 'CodeOIDC',
     '/oidc_redirect', 'RedirectOIDC',
+    '/oidc_refresh', 'RefreshOIDC',
 )
 
 
@@ -111,6 +115,7 @@ class UserPass(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO Only).
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-Username: Username as a string.
         :param Rucio-Password: SHA1 hash of the password as a string.
@@ -129,6 +134,7 @@ class UserPass(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         username = ctx.env.get('HTTP_X_RUCIO_USERNAME')
         password = ctx.env.get('HTTP_X_RUCIO_PASSWORD')
@@ -140,7 +146,7 @@ class UserPass(RucioController):
             ip = ctx.ip
 
         try:
-            result = get_auth_token_user_pass(account, username, password, appid, ip)
+            result = get_auth_token_user_pass(account, username, password, appid, ip, vo=vo)
         except AccessDenied:
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
         except RucioException as error:
@@ -201,6 +207,7 @@ class OIDC(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT', 'webui')
         auth_scope = ctx.env.get('HTTP_X_RUCIO_CLIENT_AUTHORIZE_SCOPE', "")
         audience = ctx.env.get('HTTP_X_RUCIO_CLIENT_AUTHORIZE_AUDIENCE', "")
@@ -208,8 +215,8 @@ class OIDC(RucioController):
         issuer = ctx.env.get('HTTP_X_RUCIO_CLIENT_AUTHORIZE_ISSUER', None)
         polling = ctx.env.get('HTTP_X_RUCIO_CLIENT_AUTHORIZE_POLLING', False)
         refresh_lifetime = ctx.env.get('HTTP_X_RUCIO_CLIENT_AUTHORIZE_REFRESH_LIFETIME', None)
-        auto = (auto == 'True')
-        polling = (polling == 'True')
+        auto = (auto == 'True' or auto == 'true')
+        polling = (polling == 'True' or polling == 'true')
         if refresh_lifetime == 'None':
             refresh_lifetime = None
         ip = ctx.env.get('HTTP_X_FORWARDED_FOR')
@@ -223,7 +230,7 @@ class OIDC(RucioController):
                       'polling': polling,
                       'refresh_lifetime': refresh_lifetime,
                       'ip': ip}
-            result = get_auth_oidc(account, **kwargs)
+            result = get_auth_oidc(account, vo=vo, **kwargs)
         except AccessDenied:
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot get authentication URL from Rucio Authentication Server for account %(account)s' % locals())
         except RucioException as error:
@@ -322,7 +329,7 @@ class RedirectOIDC(RucioController):
 
 class CodeOIDC(RucioController):
     """
-    IdP redirects to this endpoing with the AuthZ code
+    IdP redirects to this endpoint with the AuthZ code
     Rucio Auth server will request new token. This endpoint should be reached
     only if the request/ IdP login has been made through web browser. Then the response
     content will be in html (including the potential errors displayed).
@@ -479,14 +486,92 @@ class TokenOIDC(RucioController):
                 header('Content-Type', 'text/html')
                 render = template.render(join(dirname(__file__), '../auth_templates/'))
                 return render.auth_crash('unknown_identity')
-            # header('X-Rucio-Auth-Token', result[3].token)
-            setcookie('x-rucio-auth-token', value=result['token'].token, path='/')
-            setcookie('rucio-auth-token-created-at', value=int(time.time()), path='/')
+            # domain setting is necessary so that the token gets distributed also to the webui server
+            domain = '.'.join(urlparse.urlparse(webhome).netloc.split('.')[1:])
+            setcookie('x-rucio-auth-token', value=result['token'].token, domain=domain, path='/')
+            setcookie('rucio-auth-token-created-at', value=int(time.time()), domain=domain, path='/')
             return seeother(webhome)
         else:
             raise BadRequest()
 
 
+class RefreshOIDC(RucioController):
+    """
+    For a presented and access token which has equivalent in Rucio DB
+    (and also has refrech token in the Rucio DB) the class will attempt
+    token refresh and return a user a new refreshed token. If the presented token
+    is a result of a previous refresh happening in the last 10 min, the same token will be returned.
+    """
+
+    def OPTIONS(self):
+        """
+        HTTP Success:
+            200 OK
+
+        Allow cross-site scripting. Explicit for Authentication.
+        """
+
+        header('Access-Control-Allow-Origin', ctx.env.get('HTTP_ORIGIN'))
+        header('Access-Control-Allow-Headers', ctx.env.get('HTTP_ACCESS_CONTROL_REQUEST_HEADERS'))
+        header('Access-Control-Allow-Methods', '*')
+        header('Access-Control-Allow-Credentials', 'true')
+        header('Access-Control-Expose-Headers', 'X-Rucio-Auth-Token')
+        raise OK
+
+    @check_accept_header_wrapper(['application/octet-stream'])
+    def GET(self):
+        """
+        HTTP Success:
+            200 OK
+
+        HTTP Error:
+            401 Unauthorized
+
+        :param QUERY_STRING: the URL query string itself
+
+        :returns: "Rucio-Auth-Token" as a variable-length string header.
+        """
+
+        header('Access-Control-Allow-Origin', ctx.env.get('HTTP_ORIGIN'))
+        header('Access-Control-Allow-Headers', ctx.env.get('HTTP_ACCESS_CONTROL_REQUEST_HEADERS'))
+        header('Access-Control-Allow-Methods', '*')
+        header('Access-Control-Allow-Credentials', 'true')
+        header('Access-Control-Expose-Headers', 'X-Rucio-Auth-Token')
+
+        header('Content-Type', 'application/octet-stream')
+        header('Cache-Control', 'no-cache, no-store, max-age=0, must-revalidate')
+        header('Cache-Control', 'post-check=0, pre-check=0', False)
+        header('Pragma', 'no-cache')
+
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
+        account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
+        token = ctx.env.get('HTTP_X_RUCIO_AUTH_TOKEN')
+
+        try:
+            result = refresh_cli_auth_token(token, account, vo=vo)
+
+        except AccessDenied:
+            raise generate_http_error(401, 'CannotAuthorize', 'Cannot authorize token request.')
+        except RucioException as error:
+            raise generate_http_error(500, error.__class__.__name__, error.args[0])
+        except Exception as error:
+            print(format_exc())
+            raise InternalError(error)
+
+        if not result:
+            header('X-Rucio-Auth-Token', '')
+            header('X-Rucio-Auth-Token-Expires', '')
+            return str()
+        else:
+            if result[0] and result[1]:
+                header('X-Rucio-Auth-Token', str(result[0]))
+                header('X-Rucio-Auth-Token-Expires', str(result[1]))
+            else:
+                header('X-Rucio-Auth-Token', '')
+                header('X-Rucio-Auth-Token-Expires', '')
+            return str()
+
+
 class GSS(RucioController):
     """
     Authenticate a Rucio account temporarily via a GSS token.
@@ -516,6 +601,7 @@ class GSS(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO only).
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-AppID: Application identifier as a string.
         :param SavedCredentials: Apache mod_auth_kerb SavedCredentials.
@@ -533,6 +619,7 @@ class GSS(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         gsscred = ctx.env.get('REMOTE_USER')
         appid = ctx.env.get('HTTP_X_RUCIO_APPID')
@@ -543,7 +630,7 @@ class GSS(RucioController):
             ip = ctx.ip
 
         try:
-            result = get_auth_token_gss(account, gsscred, appid, ip)
+            result = get_auth_token_gss(account, gsscred, appid, ip, vo=vo)
         except AccessDenied:
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
 
@@ -586,6 +673,7 @@ class x509(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO only).
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-AppID: Application identifier as a string.
         :param SSLStdEnv: Apache mod_ssl SSL Standard Env Variables.
@@ -603,6 +691,7 @@ class x509(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         dn = ctx.env.get('SSL_CLIENT_S_DN')
         if not dn:
@@ -633,16 +722,16 @@ class x509(RucioController):
                 break
 
         try:
-            result = get_auth_token_x509(account, dn, appid, ip)
+            result = get_auth_token_x509(account, dn, appid, ip, vo=vo)
         except AccessDenied:
-            print('Cannot Authenticate', account, dn, appid, ip)
+            print('Cannot Authenticate', account, dn, appid, ip, vo)
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
         except IdentityError:
-            print('Cannot Authenticate', account, dn, appid, ip)
+            print('Cannot Authenticate', account, dn, appid, ip, vo)
             raise generate_http_error(401, 'CannotAuthenticate', 'No default account set for %(dn)s' % locals())
 
         if not result:
-            print('Cannot Authenticate', account, dn, appid, ip)
+            print('Cannot Authenticate', account, dn, appid, ip, vo)
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
 
         header('X-Rucio-Auth-Token', result.token)
@@ -679,6 +768,7 @@ class SSH(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO only).
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-SSH-Signature: Response to server challenge signed with SSH private key as a base64 encoded string.
         :param Rucio-AppID: Application identifier as a string.
@@ -696,6 +786,7 @@ class SSH(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         signature = ctx.env.get('HTTP_X_RUCIO_SSH_SIGNATURE')
         appid = ctx.env.get('HTTP_X_RUCIO_APPID')
@@ -712,7 +803,7 @@ class SSH(RucioController):
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with malformed signature' % locals())
 
         try:
-            result = get_auth_token_ssh(account, signature, appid, ip)
+            result = get_auth_token_ssh(account, signature, appid, ip, vo=vo)
         except AccessDenied:
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
         except RucioException as error:
@@ -758,6 +849,7 @@ class SSHChallengeToken(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO only).
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-AppID: Application identifier as a string.
         :returns: "Rucio-Auth-Token" as a variable-length string header.
@@ -774,6 +866,7 @@ class SSHChallengeToken(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         appid = ctx.env.get('HTTP_X_RUCIO_APPID')
         if appid is None:
@@ -783,7 +876,7 @@ class SSHChallengeToken(RucioController):
             ip = ctx.ip
 
         try:
-            result = get_ssh_challenge_token(account, appid, ip)
+            result = get_ssh_challenge_token(account, appid, ip, vo=vo)
         except RucioException as error:
             raise generate_http_error(500, error.__class__.__name__, error.args[0])
         except Exception as error:
@@ -827,6 +920,7 @@ class SAML(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO only)
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-Username: Username as a string.
         :param Rucio-Password: Password as a string.
@@ -845,7 +939,12 @@ class SAML(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        if not EXTRA_MODULES['onelogin']:
+            header('X-Rucio-Auth-Token', None)
+            return "SAML not configured on the server side."
+
         saml_nameid = cookies().get('saml-nameid')
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         appid = ctx.env.get('HTTP_X_RUCIO_APPID')
         if appid is None:
@@ -856,7 +955,7 @@ class SAML(RucioController):
 
         if saml_nameid:
             try:
-                result = get_auth_token_saml(account, saml_nameid, appid, ip)
+                result = get_auth_token_saml(account, saml_nameid, appid, ip, vo=vo)
             except AccessDenied:
                 raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
             except RucioException as error:
@@ -884,6 +983,11 @@ class SAML(RucioController):
         return str()
 
     def POST(self):
+
+        if not EXTRA_MODULES['onelogin']:
+            header('X-Rucio-Auth-Token', None)
+            return "SAML not configured on the server side."
+
         SAML_PATH = config_get('saml', 'config_path')
         request = ctx.env
         data = dict(param_input())
